const { registerBlockType } = wp.blocks;
const { SelectControl, PanelBody } = wp.components;
const { InspectorControls, useBlockProps } = wp.blockEditor;
const { Fragment } = wp.element;

registerBlockType("revi/revi-widget", {
    title: "Revi Widget",
    icon: "star-filled",
    category: "widgets",
    attributes: {
        widgetType: { type: "string", default: "general" }
    },
    edit: (props) => {
        const { attributes, setAttributes } = props;

        return wp.element.createElement(
            Fragment,
            {},
            wp.element.createElement(
                InspectorControls,
                {},
                wp.element.createElement(
                    PanelBody,
                    { title: "Configuración del Widget", initialOpen: true },
                    wp.element.createElement(SelectControl, {
                        label: "Tipo de Widget",
                        value: attributes.widgetType,
                        options: revi_block.widgetOptions.map(option => ({
                            label: option.type.name,
                            value: option.type.name
                        })),
                        onChange: (newValue) => setAttributes({ widgetType: newValue })
                    })
                )
            ),
            wp.element.createElement(
                "div",
                useBlockProps({ className: "revi-widget-preview" }), // 💡 Aplica la clase correctamente
                wp.element.createElement(
                    "div",
                    { className: "revi-widget-container" },
                    // Logo + Texto
                    wp.element.createElement("div", { className: "revi-widget-logo-text" },
                        wp.element.createElement("img", {
                            src: revi_block.logoUrl,
                            alt: "Revi Logo",
                            className: "revi-widget-logo"
                        }),
                        wp.element.createElement("div", {},
                            wp.element.createElement("p", { className: "revi-widget-title" }, "Revi Widget"),
                            wp.element.createElement("p", { className: "revi-widget-type" }, `${attributes.widgetType}`)
                        )
                    ),
                    // Estrellas
                    wp.element.createElement("div", { className: "revi-widget-stars" },
                        Array(5).fill(0).map((_, i) =>
                            wp.element.createElement("svg", {
                                key: i,
                                xmlns: "http://www.w3.org/2000/svg",
                                width: "20",
                                height: "20",
                                viewBox: "0 0 24 24",
                                className: "revi-star-icon"
                            },
                                wp.element.createElement("path", { d: "M12 .587l3.668 7.425 8.2 1.192-5.933 5.78 1.4 8.16L12 18.897l-7.334 3.847 1.4-8.16L.133 9.204l8.2-1.192L12 .587z" }))
                        )
                    )
                )
            )
        );
    },
    save: () => null // Se renderiza en PHP
});
