<?php
if (!defined('ABSPATH')) exit; // Evita el acceso directo

function revi_register_block()
{
    wp_register_script(
        'revi-block-editor-script',
        plugins_url('revi-block.js', __FILE__), // Asegura que la URL sea correcta
        array('wp-blocks', 'wp-editor', 'wp-components', 'wp-element', 'wp-block-editor'),
        filemtime(plugin_dir_path(__FILE__) . 'revi-block.js'),
        true
    );

    register_block_type('revi/revi-widget', array(
        'editor_script' => 'revi-block-editor-script',
        'render_callback' => 'revi_render_block_callback',
    ));
}
add_action('init', 'revi_register_block');

function revi_render_block_callback($attributes)
{
    $widgetType = isset($attributes['widgetType']) ? esc_attr($attributes['widgetType']) : 'wide';

    ob_start();
    revi_load_widget_html($widgetType);
    return ob_get_clean();
}

function revi_get_widget_preview()
{
    // Verifica que la solicitud viene desde el editor de WordPress
    if (!isset($_POST['widgetType']) || !check_ajax_referer('revi_nonce', 'security', false)) {
        wp_send_json_error(['message' => 'Acceso no autorizado']);
        wp_die();
    }

    // Obtiene el tipo de widget seleccionado
    $widgetType = sanitize_text_field($_POST['widgetType']);

    $reviWidgetsClass = new reviWidgetsClass();
    $html = $reviWidgetsClass->loadReviWidget($widgetType);

    wp_send_json_success(['html' => $html]);
}
add_action('wp_ajax_revi_get_widget_preview', 'revi_get_widget_preview');
add_action('wp_ajax_nopriv_revi_get_widget_preview', 'revi_get_widget_preview');


function revi_enqueue_block_editor_assets()
{
    wp_enqueue_script(
        'revi-block-editor-script',
        plugins_url('revi-block.js', __FILE__),
        array('wp-blocks', 'wp-element', 'wp-editor'),
        filemtime(plugin_dir_path(__FILE__) . 'revi-block.js')
    );

    $widgets = get_option('REVI_WIDGETS');
    $widgetOptions = json_decode($widgets);


    wp_localize_script('revi-block-editor-script', 'revi_block', [
        'logoUrl' => REVI_PLUGIN_URL . 'assets/img/logo-dark-pink-dot.png',
        'widgetOptions' => $widgetOptions, // Pasamos el array a JavaScript
    ]);
}

add_action('enqueue_block_editor_assets', 'revi_enqueue_block_editor_assets');

function revi_enqueue_editor_styles()
{
    add_editor_style(plugins_url('editor.css', __FILE__));
}
add_action('admin_init', 'revi_enqueue_editor_styles');
