<?php
if (!defined('ABSPATH')) exit; // Exit if accessed directly

class reviGeneralModel
{
    var $prefix;
    var $wpdb;

    public $reviCurlModel;

    public function __construct()
    {
        global $wpdb;
        $this->wpdb = $wpdb;
        $this->prefix = $this->wpdb->prefix;

        $this->reviCurlModel = new reviCurlModel();
    }

    public function getLog()
    {
        $log_file = REVI_DIR . '/logs/error.log';
        $log = file_get_contents($log_file);
        return $log;
    }

    public function resetLog()
    {
        $log_file = REVI_DIR . '/logs/error.log';
        file_put_contents($log_file, '');
    }

    public function errorLog($message)
    {
        $log_file = REVI_DIR . '/logs/error.log';
        $date = date('Y-m-d H:i:s');
        $log_entry = "[$date] $message" . PHP_EOL;

        file_put_contents($log_file, $log_entry, FILE_APPEND | LOCK_EX);
    }

    /////////// URL NEW REVIEW ///////////////

    public function getNewReviewUrl($idOrder, $medium = 1, $email = '')
    {
        $wc_order = wc_get_order($idOrder);

        if (!empty($wc_order)) {
            return '';
        }

        $iso_country = $wc_order->get_shipping_country();
        $lang = $this->getOrderLang($idOrder, $iso_country);
        $customer_firstname = $wc_order->get_billing_first_name();
        $customer_lastname = $wc_order->get_billing_last_name();
        $email = $wc_order->get_billing_email();
        $currency = $wc_order->get_currency();
        $total_paid = $wc_order->get_total();
        $date_order = $wc_order->get_date_created()->date('Y-m-d H:i:s');

        return $this->reviCurlModel->reviCURL('createNewReviewLink', 'POST', [
            'id_external_order' => $idOrder,
            'iso_code' => $lang,
            'customer_firstname' => $customer_firstname,
            'customer_lastname' => $customer_lastname,
            'email' => $email,
            'currency' => $currency,
            'total_paid' => $total_paid,
            'date_order' => $date_order,
            'medium' => $medium, // 1 Popup | 2 Order history
        ]);
    }

    private function getOrderLang($idOrder, $iso_country)
    {
        $lang = get_post_meta($idOrder, 'wpml_language', true);
        if (!empty($lang) && strlen($lang) >= 2) {
            $lang = substr($lang, 0, 2);
        } else if (!empty($iso_country) && strlen($iso_country) >= 2) {
            $lang = substr($iso_country, 0, 2);
        } else {
            $lang = get_option('REVI_SELECTED_LANGUAGE');
        }
        return $lang;
    }



    /////////////// UPDATE CONFIGURATION ///////////////

    public function updateConfiguration()
    {
        $result = $this->reviCurlModel->reviCURL('connectedPlatformSettings', 'GET');

        if (!isset($result->settings)) {
            update_option('REVI_SUBSCRIPTION', '0');
            return false;
        }

        $settings = $result->settings;

        if ($settings->widgets) {

            // Podemos guardar todo en un solo campo
            update_option('REVI_WIDGETS', json_encode($settings->widgets));

            foreach ($settings->widgets as $widget) {
                update_option('REVI_WIDGET_' . $widget->type->name, $widget->hashID);
            }
        }

        update_option('REVI_LANG', $settings->default_language);
        update_option('REVI_SUBSCRIPTION', $settings->subscription_plan);

        if (get_option('REVI_SELECTED_LANGUAGE') == '') {
            update_option('REVI_SELECTED_LANGUAGE', $settings->default_language);
        }
        update_option('REVI_ACTIVE_LANGUAGES', json_encode($settings->languages));

        return true;
    }

    public function sendModuleVersion()
    {
        $plugin_data = get_plugin_data(WP_PLUGIN_DIR . '/revi-io-customer-and-product-reviews/revi.php');
        $plugin_version = $plugin_data['Version'];

        $this->reviCurlModel->reviCURL('moduleVersion', 'POST', ['version' => $plugin_version]);

        update_option('REVI_MODULE_VERSION', $plugin_version);
    }

    public function checkModuleMessage()
    {
        $plugin_data = get_plugin_data(WP_PLUGIN_DIR . '/revi-io-customer-and-product-reviews/revi.php');
        $plugin_version = $plugin_data['Version'];

        return $this->reviCurlModel->reviCURL('checkModuleMessage', 'POST', ['version' => $plugin_version]);
    }

    public function getSubscriptionName()
    {
        $subscription = get_option('REVI_SUBSCRIPTION');

        $subscriptionName = 'Free';
        if ($subscription == 1) {
            $subscriptionName = 'Lite';
        } else if ($subscription == 2) {
            $subscriptionName = 'Pro';
        } else if ($subscription == 3) {
            $subscriptionName = 'Premium';
        } else if ($subscription == 4) {
            $subscriptionName = 'Unlimited';
        }

        return $subscriptionName;
    }
}
