<?php
if (!defined('ABSPATH')) exit; // Exit if accessed directly

class reviOrdersModel
{
    var $prefix;
    var $wpdb;

    public $reviDatabaseModel;
    public $reviCurlModel;
    public $reviGeneralModel;
    public $reviProductsModel;

    public function __construct()
    {
        global $wpdb;
        $this->wpdb = $wpdb;
        $this->prefix = $this->wpdb->prefix;

        $this->reviDatabaseModel = new reviDatabaseModel();
        $this->reviCurlModel = new reviCurlModel();
        $this->reviGeneralModel = new reviGeneralModel();
        $this->reviProductsModel = new reviProductsModel();
    }

    ////////////////////// REVI ORDERS ///////////////////////
    public function addReviOrder($idOrder)
    {
        $date_sent = date('Y-m-d H:i:s');
        if (!$this->checkReviOrderExist($idOrder)) {
            $this->insertReviOrder($idOrder, $date_sent);
        } else {
            $this->updateReviOrders($idOrder, $date_sent);
        }
    }

    public function insertReviOrder($idOrder, $date_sent = null)
    {
        if (empty($date_sent)) {
            $date_sent = date('Y-m-d H:i:s');
        }
        $sql = "INSERT INTO `{$this->prefix}revi_orders` (id_order, date_sent) VALUES ('" . $idOrder . "', '" . $date_sent . "')";
        $this->wpdb->query($sql);
    }

    public function updateReviOrders($idOrder, $date_sent = null)
    {
        if (empty($date_sent)) {
            $date_sent = date('Y-m-d H:i:s');
        }
        $sql = "UPDATE `{$this->prefix}revi_orders` SET date_sent = '" . $date_sent . "' WHERE id_order = '" . $idOrder . "'";
        $this->wpdb->query($sql);
    }

    public function checkReviOrderExist($idOrder)
    {
        $sql = "SELECT id_order FROM `{$this->prefix}revi_orders` WHERE id_order = '" . $idOrder . "'";
        $result = $this->wpdb->get_row($sql);
        if (!empty($result->id_order)) {
            return $result->id_order;
        } else {
            return 0;
        }
    }

    ////////////////////// PS ORDERS ///////////////////////

    /*
    * Devuelve Order
    */
    public function getOrder($idOrder)
    {
        $this->wpdb->query('SET SQL_BIG_SELECTS = 1');

        // Verificar si las nuevas tablas de WooCommerce existen
        if ($this->reviDatabaseModel->checkNewWoocommerceTables()) {
            // Verificar si el pedido existe en las nuevas tablas
            $sqlNew = "SELECT WCO.ID as id_order, WCO.date_created_gmt as date_order
                        FROM {$this->prefix}wc_orders WCO
                        WHERE WCO.ID = '$idOrder'
                        LIMIT 1";

            $order = $this->wpdb->get_row($sqlNew);

            // Si encontramos el pedido, lo retornamos
            if ($order) {
                return $order;
            }
        }

        // Si no existen las nuevas tablas o no hay datos para ese pedido, usar las tablas anteriores de WordPress
        $sqlOld = "SELECT P.ID as id_order, P.post_date_gmt as date_order
                    FROM {$this->prefix}posts P 
                    WHERE P.post_type IN ('shop_order', 'shop_order_placehold') 
                    AND P.ID = '$idOrder'
                    LIMIT 1";

        return $this->wpdb->get_row($sqlOld);
    }

    /*
    * Devuelve orders que no estén en revi_orders
    */
    public function getPendingOrders($limit = 100)
    {
        $this->wpdb->query('SET SQL_BIG_SELECTS = 1');

        // Verificar si las nuevas tablas de WooCommerce existen
        if ($this->reviDatabaseModel->checkNewWoocommerceTables()) {
            // Usar las nuevas tablas de WooCommerce
            $sqlNew = "SELECT WCO.ID as id_order
                        FROM {$this->prefix}wc_orders WCO
                        LEFT JOIN `{$this->prefix}revi_orders` RO ON RO.id_order = WCO.ID
                        WHERE WCO.date_created_gmt > NOW() - INTERVAL 365 DAY 
                        -- AND type IN ('shop_order', 'shop_order_placehold')
                        AND (RO.id_order IS NULL OR RO.date_sent < WCO.date_updated_gmt)
                        ORDER BY WCO.date_created_gmt DESC
                        LIMIT $limit";

            $orders = $this->wpdb->get_results($sqlNew);

            // Si encontramos pedidos en las nuevas tablas, los retornamos
            if ($orders) {
                return $orders;
            }
        }

        // Si no existen las nuevas tablas o no hay datos en ellas, usar las tablas anteriores de WordPress
        $sqlOld = "SELECT P.ID as id_order
                    FROM {$this->prefix}posts P 
                    LEFT JOIN `{$this->prefix}revi_orders` RO ON RO.id_order = P.ID
                    WHERE P.post_date_gmt > NOW() - INTERVAL 365 DAY 
                    AND P.post_type IN ('shop_order', 'shop_order_placehold') 
                    AND (RO.id_order IS NULL OR RO.date_sent < P.post_modified_gmt)
                    GROUP BY P.ID 
                    ORDER BY P.post_date_gmt DESC
                    LIMIT $limit";

        return $this->wpdb->get_results($sqlOld);
    }

    public function getNumPendingOrdersLeft()
    {
        $this->wpdb->query('SET SQL_BIG_SELECTS = 1');

        // Verificar si las nuevas tablas de WooCommerce existen
        if ($this->reviDatabaseModel->checkNewWoocommerceTables()) {
            // Usar las nuevas tablas de WooCommerce
            $sqlNew = "SELECT COUNT(WCO.ID) as num_orders
                        FROM {$this->prefix}wc_orders WCO
                        LEFT JOIN `{$this->prefix}revi_orders` RO ON RO.id_order = WCO.ID
                        WHERE WCO.date_created_gmt > NOW() - INTERVAL 365 DAY 
                        AND (RO.id_order IS NULL OR RO.date_sent < WCO.date_updated_gmt)";

            $numOrders = $this->wpdb->get_row($sqlNew);

            // Si encontramos pedidos en las nuevas tablas, los retornamos
            if ($numOrders) {
                return $numOrders->num_orders;
            }
        }

        // Si no existen las nuevas tablas o no hay datos en ellas, usar las tablas anteriores de WordPress
        $sqlOld = "SELECT COUNT(P.ID) as num_orders
                    FROM {$this->prefix}posts P 
                    LEFT JOIN `{$this->prefix}revi_orders` RO ON RO.id_order = P.ID
                    WHERE P.post_date_gmt > NOW() - INTERVAL 365 DAY 
                    AND P.post_type IN ('shop_order', 'shop_order_placehold') 
                    AND (RO.id_order IS NULL OR RO.date_sent < P.post_modified_gmt)";

        $numOrders = $this->wpdb->get_row($sqlOld);

        return $numOrders->num_orders;
    }


    public function getOrderStatus($idOrder)
    {
        // Verificar si las nuevas tablas de WooCommerce existen
        if ($this->reviDatabaseModel->checkNewWoocommerceTables()) {
            $sql = "SELECT WCO.ID as id_order, WCO.status, WCO.date_updated_gmt as date_status_upd
                        FROM {$this->prefix}wc_orders WCO
                        WHERE WCO.ID = '$idOrder'";
        } else {
            $sql = "SELECT P.ID as id_order, P.post_status as status, P.post_modified_gmt as date_status_upd
                        FROM {$this->prefix}posts P
                        WHERE P.ID = '$idOrder'
                        AND P.post_type IN ('shop_order', 'shop_order_placehold')";
        }

        $orderStatusArray = $this->wpdb->get_row($sql);

        if (empty($orderStatusArray)) {
            return [
                'status' => 1,
                'date_status_upd' => null
            ];
        }

        // Obtener arrays de estados válidos y cancelados
        $validStatusArray = $this->getOrderStatusValidArray();
        $cancelledStatusArray = $this->getOrderStatusCancelledArray();

        if (in_array($orderStatusArray->status, $cancelledStatusArray)) {
            return [
                'status' => 0,
                'date_status_upd' => $orderStatusArray->date_status_upd
            ];
        }

        if (in_array($orderStatusArray->status, $validStatusArray)) {
            return [
                'status' => 2,
                'date_status_upd' => $orderStatusArray->date_status_upd
            ];
        }

        return [
            'status' => 1,
            'date_status_upd' => null
        ];
    }

    public function getOrderStatusValidArray()
    {
        $selectedStatus = get_option('REVI_ORDER_STATUSES');
        if (empty($selectedStatus)) {
            $selectedStatus = ['wc-completed'];
        }
        return $selectedStatus;
    }

    public function getOrderStatusCancelledArray()
    {
        return ['wc-cancelled'];
    }




    ///////////////////////////// ORDER PRODUCTS ////////////////////

    public function getOrderProducts($idOrder)
    {
        $wcOrder = wc_get_order($idOrder);

        $orderProductsArray = [];
        if (empty($wcOrder)) {
            $this->reviGeneralModel->errorLog('getOrderProducts | Order not found for ID: ' . $idOrder);
            return null;
        }

        $orderItems = $wcOrder->get_items('line_item'); // Solo productos

        if (empty($orderItems)) {
            $this->reviGeneralModel->errorLog('getOrderProducts | Order items not found for ID: ' . $idOrder);
            return null;
        }

        foreach ($orderItems as $item_id => $item) {

            if (empty($item)) {
                $this->reviGeneralModel->errorLog('getOrderProducts | Item not found | id_order: ' . $idOrder . ' | item_id: ' . $item_id);
                continue;
            }

            $product = $item->get_product();
            if (empty($product)) {
                $this->reviGeneralModel->errorLog('getOrderProducts | Product not found | id_order: ' . $idOrder . ' | item_id: ' . $item_id);
                continue;
            }

            $idExternalProduct = $this->reviProductsModel->get_id_main_product($item->get_product_id());

            if (empty($idExternalProduct)) {
                $this->reviGeneralModel->errorLog('getOrderProducts | idExternalProduct Not found | id_order: ' . $idOrder . ' | id_product: ' . $item->get_product_id());
                continue;
            }

            $idExternalProductCombination = $this->getProductVariation($product);

            $taxes = (float) $item->get_total_tax();
            $productPriceWithoutTax = (float) $product->get_price();
            $productPriceWithTax = $productPriceWithoutTax + $taxes;
            $qty = $item->get_quantity();
            $totalPrice = $productPriceWithTax * $qty;
            $totalPaidWithoutTaxes = $totalPrice - $taxes;

            $vat = 0;
            if ($item->get_total_tax() > 0) {
                if (!empty($product->get_price())) {
                    $vat = round((($totalPrice / $productPriceWithoutTax) - 1) * 100, 2);
                }
            }
            if (is_infinite($vat)) {
                $vat = 0;
            }

            $orderProductsArray[] = [
                'id_external_product' => $idExternalProduct,
                'id_external_product_combination' => $idExternalProductCombination ?? null,
                'quantity' => $qty,
                'price_unit' => $productPriceWithTax,
                'taxes' => $taxes,
                'vat' => $vat,
                'total_paid_without_taxes' => $totalPaidWithoutTaxes,
                'total_price' => $productPriceWithTax * $qty,
            ];
        }

        return $orderProductsArray;
    }

    /*
    * object $product
    *
    * return int
    */
    public function getProductVariation($product)
    {
        // Obtener la versión de WooCommerce
        $wc_version = defined('WC_VERSION') && WC_VERSION ? WC_VERSION : null;

        // Only for product variation
        if ($product->is_type('variation')) {
            // Comprobar si la versión de WooCommerce es menor a 3.0
            if (version_compare($wc_version, '3.0', '<')) {
                // Usar get_variation_id para versiones anteriores
                return $product->get_variation_id();
            } else {
                // Usar get_id para versiones 3.0 y superiores
                return $product->get_id();
            }
        }
    }

    public function parseOrderData($idOrder)
    {
        $wcOrder = wc_get_order($idOrder);
        if (empty($wcOrder)) {
            $this->reviGeneralModel->errorLog('parseOrderData | Order not found for ID: ' . $idOrder);
            return null;
        }

        // Check if the order is a refund object
        if ($wcOrder instanceof WC_Order_Refund) {
            return null; // Skip the refund object
        }

        $email = $wcOrder->get_billing_email();

        if (empty($email) || !filter_var($email, FILTER_VALIDATE_EMAIL)) {
            $this->reviGeneralModel->errorLog('parseOrderData | Invalid email for order ID: ' . $idOrder);
            return null;
        }


        $statusData = $this->getOrderStatus($idOrder);

        $ordersData = [
            'id_external_order' => $idOrder,
            'customer_firstname' => $wcOrder->get_billing_first_name(),
            'customer_lastname' => $wcOrder->get_billing_last_name(),
            'email' => $email,
            'currency' => $wcOrder->get_currency(),
            'shipping_cost' => $wcOrder->get_shipping_total(),
            'total_discount' => $wcOrder->get_discount_total(),
            'date_order' => $wcOrder->get_date_created()->date('Y-m-d H:i:s'),
            'status' => $statusData['status'],
            'date_status_upd' => $statusData['date_status_upd'],
        ];

        $iso_country = $wcOrder->get_shipping_country();
        if (!empty($iso_country)) {
            $ordersData['iso_country'] = $iso_country;
        }

        $lang = get_post_meta($idOrder, 'wpml_language', true);

        if (!empty($lang) && strlen($lang) >= 2) {
            $lang = substr($lang, 0, 2);
        } else if (!empty($iso_country) && strlen($iso_country) >= 2) {
            $lang = substr($iso_country, 0, 2);
        } else {
            $lang = get_option('REVI_SELECTED_LANGUAGE');
        }
        $ordersData['iso_code'] = $lang;


        $total_paid = $wcOrder->get_total();
        $total_tax = $wcOrder->get_total_tax();
        $total_price_without_taxes = $total_paid - $total_tax;

        $ordersData['total_paid'] = $total_paid;
        $ordersData['taxes'] = $total_tax;

        if ($total_tax && $total_price_without_taxes) { //para que no sea dividido por infinito
            $ordersData['vat'] = round((($total_paid / $total_price_without_taxes) - 1) * 100, 0);
        } else {
            $ordersData['vat'] = 0;
        }
        if (is_infinite($ordersData['vat'])) {
            unset($ordersData['vat']);
        }

        return $ordersData;
    }

    public function sendOrderProducts($idOrder)
    {
        $orderProducts = $this->getOrderProducts($idOrder);

        if (empty($orderProducts)) {
            $this->reviGeneralModel->errorLog('sendOrderProducts | Order products not found for ID: ' . $idOrder);
            return false;
        }

        // WEE_TODO
        // quitar los que ya hayan sido enviados previamente
        $productIdsArray = [];
        foreach ($orderProducts as $orderProduct) {
            $productIdsArray[] = $orderProduct['id_external_product'];
        }

        $products = $this->reviProductsModel->parseProductsWithLanguages($productIdsArray);
        $this->reviProductsModel->sendProducts($products);

        // Parsemos los order products para enviarlos
        $orderProductsParsed = $this->parseOrderProducts($orderProducts);

        $result = $this->reviCurlModel->reviCURL('orderProducts', 'POST', [
            'id_external_order' => $idOrder,
            'order_products' => json_encode($orderProductsParsed),
            'delete_products' => true,
        ]);

        if (empty($result) || !isset($result->success)) {
            $this->reviGeneralModel->errorLog('Error reviCURL:orderProducts: ' . json_encode($result));
            return false;
        }

        foreach ($orderProducts as $product) {
            $this->reviProductsModel->addReviProduct([
                'id_product' => $product['id_external_product']
            ], date('Y-m-d H:i:s'));
        }
    }

    public function parseOrderProducts($orderProducts)
    {
        $orderProductsParsed = [];
        $j = 0;
        foreach ($orderProducts as $product) {
            $orderProductsParsed[$j]['id_external_product'] = $product['id_external_product'];
            $orderProductsParsed[$j]['id_external_product_combination'] = $product['id_external_product_combination'] ?? null;
            $orderProductsParsed[$j]['quantity'] = $product['quantity'];
            $orderProductsParsed[$j]['price_unit'] = $product['price_unit'];
            $orderProductsParsed[$j]['vat'] = $product['vat'];
            $orderProductsParsed[$j]['total_price'] = $product['total_price'];
            $orderProductsParsed[$j]['taxes'] = (int) $product['total_price'] - (int) $product['total_paid_without_taxes'];
            ++$j;
        }

        return $orderProductsParsed;
    }

    public function sendPendingOrders($limit = 100)
    {
        try {
            $orders = $this->getPendingOrders($limit);

            if (empty($orders)) {
                return 0;
            }

            $ordersData = [];
            foreach ($orders as $order) {

                $parsedOrder = $this->parseOrderData($order->id_order);

                if (empty($parsedOrder)) {
                    continue;
                }

                $ordersData[] = $parsedOrder;

                $this->sendOrderProducts($order->id_order);
            }

            if (empty($ordersData)) {
                return 0;
            }

            // Enviamos los pedidos
            $result = $this->reviCurlModel->reviCURL('orders', 'POST', [
                'orders' => json_encode($ordersData),
            ]);

            if (empty($result) || !isset($result->success)) {
                $this->reviGeneralModel->errorLog('Error reviCURL:orders: ' . json_encode($result));
                return -1;
            }

            foreach ($ordersData as $orderData) {
                $this->addReviOrder($orderData['id_external_order']);
            }

            return count($ordersData);
        } catch (Exception $e) {
            $this->reviGeneralModel->errorLog('Error sendAllOrders: ' . $e->getMessage());
            return -1;
        }
    }

    public function sendOrder($idOrder)
    {
        $parsedOrder = $this->parseOrderData($idOrder);

        if (empty($parsedOrder)) {
            return false;
        }

        $this->sendOrderProducts($idOrder);

        $ordersData[] = $parsedOrder;

        // Enviamos los pedidos
        $result = $this->reviCurlModel->reviCURL('orders', 'POST', [
            'orders' => json_encode($ordersData),
        ]);

        if (!empty($result) && isset($result->success)) {
            $this->addReviOrder($idOrder);
        }
    }

    public function sendOrderStatus($idOrder)
    {
        if (empty($idOrder)) {
            return;
        }

        $orderStatus = $this->getOrderStatus($idOrder);

        $ordersData[0] = [
            'id_external_order' => $idOrder,
            'status' => $orderStatus['status'],
            'date_status_upd' => date('Y-m-d H:i:s'),
        ];

        $this->reviCurlModel->reviCURL('ordersStatus', 'POST', [
            'orders' => json_encode($ordersData),
        ]);
    }
}
