<?php
if (!defined('ABSPATH')) exit; // Exit if accessed directly

class reviProductsModel
{
    var $prefix;
    var $wpdb;

    public $reviCurlModel;
    public $reviGeneralModel;

    public function __construct()
    {
        global $wpdb;
        $this->wpdb = $wpdb;
        $this->prefix = $this->wpdb->prefix;

        $this->reviCurlModel = new reviCurlModel();
        $this->reviGeneralModel = new reviGeneralModel();
    }


    ///////////////////////////// REVI PRODUCTS ////////////////////

    public function getReviProduct($idProduct)
    {
        $sql = "SELECT * FROM `{$this->prefix}revi_products` WHERE id_product = '" . $idProduct . "'";
        $product = $this->wpdb->get_row($sql);

        if (isset($product->id_product)) {
            $product->avg_rating = round($product->avg_rating, 2);
            return $product;
        } else {
            $product = new StdClass();
            $product->id_product = $idProduct;
            $product->num_reviews = 0;
            $product->avg_rating = 0;
            return $product;
        }
    }

    public function addReviProduct($productData, $dateSent = null)
    {
        if (empty($productData['id_product'])) {
            $this->reviGeneralModel->errorLog('Error addReviProduct: id_product is empty');
            return false;
        }

        if (!$this->checkReviProductExist($productData['id_product'])) {
            return $this->insertReviProduct($productData, $dateSent);
        } else {
            return $this->updateReviProduct($productData, $dateSent);
        }
    }

    public function checkReviProductExist($idProduct)
    {
        $sql = "SELECT id_product FROM `{$this->prefix}revi_products` WHERE id_product = '" . $idProduct . "'";
        return $this->wpdb->get_var($sql);
    }

    public function insertReviProduct($productData, $dateSent)
    {
        if (empty($productData['num_reviews'])) {
            $productData['num_reviews'] = 0;
        }
        if (empty($productData['avg_rating'])) {
            $productData['avg_rating'] = 5;
        }

        $sql = "INSERT INTO `{$this->prefix}revi_products` (id_product, num_reviews, avg_rating, date_sent) VALUES ('" . $productData['id_product'] . "', '" . $productData['num_reviews'] . "', '" . $productData['avg_rating'] . "', '" . $dateSent . "')";
        $this->wpdb->query($sql);
    }

    public function updateReviProduct($productData, $dateSent)
    {
        if ($dateSent) {
            $sql = "UPDATE `{$this->prefix}revi_products` SET date_sent = '" . $dateSent . "'";
        } else {
            $sql = "UPDATE `{$this->prefix}revi_products` SET num_reviews = '" . $productData['num_reviews'] . "', avg_rating = '" . $productData['avg_rating'] . "'";
        }
        $sql .= " WHERE id_product = '" . $productData['id_product'] . "'";
        $this->wpdb->query($sql);
    }

    ///////////////////////////// PRODUCTS ////////////////////

    //Busca el main product de WPML
    public function get_id_main_product($idProduct)
    {
        try {
            // WPML
            if (REVI_PLUGIN_LANGUAGE === 'wpml') {
                if (function_exists('wpml_get_content_translation_source_id')) {
                    $source_id = wpml_get_content_translation_source_id($idProduct, 'post_product');
                    if ($source_id) {
                        return $source_id;
                    }
                }

                // Fallback SQL si no está disponible la función
                if (function_exists('icl_object_id')) {
                    $prepared_id = intval($idProduct);
                    $table = $this->wpdb->prefix . 'icl_translations';

                    $trid = $this->wpdb->get_row(
                        $this->wpdb->prepare(
                            "SELECT trid FROM $table WHERE element_type = 'post_product' AND element_id = %d",
                            $prepared_id
                        )
                    );

                    if (!empty($trid->trid)) {
                        $translation_group = $this->wpdb->get_row(
                            $this->wpdb->prepare(
                                "SELECT element_id FROM $table WHERE trid = %d AND source_language_code IS NULL",
                                $trid->trid
                            )
                        );

                        if (!empty($translation_group->element_id)) {
                            return intval($translation_group->element_id);
                        }
                    }
                }
            }

            // Polylang
            if (REVI_PLUGIN_LANGUAGE === 'polylang' && function_exists('pll_get_post_translations')) {
                $translations = pll_get_post_translations($idProduct);
                $default_lang = function_exists('pll_default_language') ? pll_default_language() : null;

                if ($default_lang && isset($translations[$default_lang])) {
                    return $translations[$default_lang];
                }
            }

            // Fallback: devolver el propio ID si no se pudo resolver
            return $idProduct;
        } catch (Exception $e) {
            $this->reviGeneralModel->errorLog('Error get_id_main_product: ' . $e->getMessage());
            return $idProduct;
        }
    }

    public function get_product_language($idProduct)
    {
        $lang = null;
        if (array_key_exists('wpml_post_language_details', $GLOBALS['wp_filter'])) {
            $language_data = apply_filters('wpml_post_language_details', null, $idProduct);

            if (isset($language_data['language_code'])) {
                $lang = $language_data['language_code'];
            }
        }

        if (!empty($lang) && strlen($lang) > 0) {
            $lang = reviParseLang($lang);
            return $lang;
        } else {
            return get_option('REVI_SELECTED_LANGUAGE');
        }
    }

    /**
     * Obtiene los productos que no se han enviado a Revi.
     **/
    public function getNumProductsNotSent()
    {
        //WORDPRESS DA FALLO DE BIG QUERYS A VECES, CON ESTO SE SOLUCIONA
        $this->wpdb->query('SET SQL_BIG_SELECTS = 1');

        $sql = "SELECT COUNT(P.ID) as num_products 
        FROM " . $this->prefix . "posts P 
        LEFT JOIN `{$this->prefix}revi_products` RP ON P.ID = RP.id_product
        WHERE RP.id_product IS NULL
        AND P.post_type = 'product'";

        return $this->wpdb->get_row($sql);
    }

    /**
     * Obtiene los productos que no se han enviado a Revi.
     **/
    public function getProductsNotSent($limit)
    {
        //WORDPRESS DA FALLO DE BIG QUERYS A VECES, CON ESTO SE SOLUCIONA
        $this->wpdb->query('SET SQL_BIG_SELECTS = 1');

        $sql = "SELECT P.ID as id_product
        FROM {$this->prefix}posts P
        LEFT JOIN {$this->prefix}revi_products RP ON P.ID = RP.id_product
        WHERE RP.id_product IS NULL
        AND P.post_type = 'product'
        LIMIT $limit";

        return $this->wpdb->get_results($sql, ARRAY_A);
    }

    /**
     * Obtiene los productos que se han actualizado tras haberlos sincronizado con Revi.
     **/
    public function getNumProductsUpdated()
    {
        //WORDPRESS DA FALLO DE BIG QUERYS A VECES, CON ESTO SE SOLUCIONA
        $this->wpdb->query('SET SQL_BIG_SELECTS = 1');

        $sql = "SELECT COUNT(P.ID) as num_products 
        FROM " . $this->prefix . "posts P 
        LEFT JOIN `{$this->prefix}revi_products` RP ON RP.id_product = P.ID
        WHERE P.post_type = 'product' 
        AND P.post_modified_gmt > RP.date_sent";

        return $this->wpdb->get_row($sql);
    }

    /**
     * Obtiene los productos que se han actualizado tras haberlos sincronizado con Revi.
     **/
    public function getProductsUpdated($limit)
    {
        //WORDPRESS DA FALLO DE BIG QUERYS A VECES, CON ESTO SE SOLUCIONA
        $this->wpdb->query('SET SQL_BIG_SELECTS = 1');

        $sql = "SELECT P.ID as id_product
        FROM " . $this->prefix . "posts P 
        LEFT JOIN `{$this->prefix}revi_products` RP ON RP.id_product = P.ID
        WHERE P.post_type = 'product' 
        AND P.post_modified_gmt > RP.date_sent
        LIMIT $limit";

        return $this->wpdb->get_results($sql, ARRAY_A);
    }

    public function sendNumProductsLeft()
    {
        $numProductsLeft = (int) $this->getNumProductsLeft();
        $this->reviCurlModel->reviCURL('productsLeft', 'POST', ['num_products_left' => $numProductsLeft]);
    }

    /**
     * Num products left
     **/
    public function getNumProductsLeft()
    {
        $numProductsNotSent = $this->getNumProductsNotSent();
        $numProductsNotUpdated = $this->getNumProductsUpdated();
        return $numProductsNotSent->num_products + $numProductsNotUpdated->num_products;
    }

    /**
     * Obtener todos los ids de productos para sincronizar con Revi
     * @param int $limit
     * @return array
     **/
    public function getProductsIdsToSend($limit)
    {
        $productsNotSent = $this->getProductsNotSent($limit);
        $productsUpdated = $this->getProductsUpdated($limit);

        $products = array_merge($productsNotSent, $productsUpdated);

        if (empty($products)) {
            return [];
        }

        // Extraer solo los IDs
        return array_column($products, 'id_product');
    }

    private function parseProduct($idProduct)
    {
        $wcProduct = wc_get_product($idProduct);
        if (empty($wcProduct)) {
            $this->reviGeneralModel->errorLog('Error parseProduct: Product not found for ID ' . $idProduct);
            return null;
        }

        // Obtener idioma del producto
        $isoCode = $this->get_product_language($idProduct);

        // Obtener URL correcta según el idioma
        $url = $wcProduct->get_permalink(); // fallback

        if (REVI_PLUGIN_LANGUAGE === 'wpml' && function_exists('wpml_post_language_details')) {
            $lang = apply_filters('wpml_post_language_details', null, $idProduct);
            if (!empty($lang['language_code'])) {

                $url = apply_filters('wpml_permalink', get_permalink($idProduct), $lang['language_code']);
            }
        } elseif (REVI_PLUGIN_LANGUAGE === 'polylang' && function_exists('pll_get_post_language')) {
            $lang = pll_get_post_language($idProduct, 'slug');
            if ($lang) {
                $url = pll_get_permalink($idProduct, $lang);
            }
        }

        // Imagen destacada
        $image_array = wp_get_attachment_image_src(get_post_thumbnail_id($idProduct), 'full');
        $image_url = (!empty($image_array[0])) ? $image_array[0] : '';

        $auxProduct = [
            'id_wp_product' => $idProduct,
            'price' => $wcProduct->get_price(),
            'name' => $wcProduct->get_name(),
            'iso_code' => $isoCode,
            'url' => $url,
            'image_url' => $image_url,
            'ean' => $this->getCombinationMetaValue($idProduct, 'gtin', null),
            'brand' => $this->getCombinationMetaValue($idProduct, 'brand', null),
            'stock' => $wcProduct->get_stock_quantity(),
        ];

        // Combinaciones / variaciones
        if (method_exists($wcProduct, 'get_available_variations')) {
            $variations = $wcProduct->get_available_variations();

            $combinations = [];
            foreach ($variations as $variation) {
                if (!empty($variation['variation_is_active'])) {
                    $combinations[] = [
                        'id_external_product_combination' => $variation['variation_id'],
                        'sku' => $this->getCombinationMetaValue($variation['variation_id'], 'sku', $variation),
                        'brand' => $this->getCombinationMetaValue($variation['variation_id'], 'brand', $variation),
                        'ean' => $this->getCombinationMetaValue($variation['variation_id'], 'gtin', $variation),
                        'image_url' => $variation['image']['url'] ?? '',
                    ];
                }
            }

            if (!empty($combinations)) {
                $auxProduct['combinations'] = $combinations;
            }
        }

        // ID externo principal
        $auxProduct['id_external_product'] = $this->get_id_main_product($idProduct);

        // Sincronizar descripción
        if (get_option('REVI_SYNC_PRODUCT_DESCRIPTION')) {
            $auxProduct['description'] = $wcProduct->get_description();
        }

        return $auxProduct;
    }



    function getCombinationMetaValue($idProduct, $metaKeyName, $variation)
    {
        if (isset($variation["{$metaKeyName}"]) && !empty($variation["{$metaKeyName}"]) && strlen($variation["{$metaKeyName}"]) > 0) {
            return $variation["{$metaKeyName}"];
        }

        // if (!empty($product->{$metaKeyName}) && strlen($product->{$metaKeyName}) > 0) {
        //     return $product->{$metaKeyName};
        // }

        $yith_name = $this->getYithProductMetaData($idProduct, $metaKeyName);
        if (!empty($yith_name) && strlen($yith_name) > 0) {
            return $yith_name;
        }

        if ($metaKeyName == 'brand') {
            $constant = PRODUCT_BRAND;
            $capital_name = 'BRAND';
        } else if ($metaKeyName == 'gtin') {
            $constant = PRODUCT_EAN;
            $capital_name = 'EAN';
        }

        if (!empty($constant)) {
            $wcProduct = wc_get_product($idProduct);

            foreach ($constant as $possible_meta_name) {
                if (strlen($wcProduct->get_attribute($possible_meta_name))) {
                    return $wcProduct->get_attribute($possible_meta_name);
                } else if (strlen(get_post_meta($idProduct, $possible_meta_name, true))) {
                    return get_post_meta($idProduct, $possible_meta_name, true);
                } else if (strlen($wcProduct->get_meta($possible_meta_name))) { // PLUGIN Product GTIN (EAN, UPC, ISBN) for WooCommerce
                    return $wcProduct->get_meta($possible_meta_name);
                } else {
                    $terms = wp_get_post_terms($wcProduct->get_id(), $possible_meta_name);
                    if (!empty($terms) && is_array($terms)) {
                        $term = array_shift($terms); // Obtener el primer término correctamente
                        if (!empty($term->name) && strlen($term->name) > 0) {
                            return $term->name;
                        }
                    }
                }
            }


            // Algunas tiendas utilizan numeraciones raras, tipo EAN-1 EAN-55, EAN-200, etc
            for ($i = 0; $i < 200; $i++) {
                if (strlen($wcProduct->get_attribute("$capital_name-" . $i))) {
                    return $wcProduct->get_attribute("$capital_name-" . $i);
                } else if (strlen($wcProduct->get_attribute("$capital_name-" . $i))) {
                    return $wcProduct->get_attribute("$capital_name-" . $i);
                }
            }
        }

        return '';
    }

    function getYithProductMetaData($idProduct, $meta_key)
    {
        $sql = "SELECT meta_value FROM " . $this->prefix . "postmeta WHERE post_id = '" . $idProduct . "' AND meta_key = 'yith_wcgpf_product_feed_configuration'";
        $data_value = $this->wpdb->get_results($sql);

        if (!empty($data_value)) {
            $unserialized_data = unserialize($data_value[0]->meta_value);

            if (!empty($unserialized_data[$meta_key])) {
                return $unserialized_data[$meta_key];
            }
        }

        return '';
    }

    public function syncProductRatings()
    {
        try {
            $numProducts = 0;

            $response = $this->reviCurlModel->reviCURL('productsRatings', 'GET');

            if (!isset($response->success)) {
                $this->reviGeneralModel->errorLog('Error syncProductRatings reviCURL:result: ' . json_encode($response));
                return -1;
            }

            $products = $response->products;

            if (empty($products)) {
                return 0;
            }

            foreach ($products as $product) {

                if (empty($product->id_external_product)) {
                    continue;
                }

                $this->addReviProduct([
                    'id_product' => $product->id_external_product,
                    'num_reviews' => $product->num_reviews,
                    'avg_rating' => $product->avg_rating,
                ]);
                $numProducts++;
            }

            return $numProducts;
        } catch (Exception $e) {
            $this->reviGeneralModel->errorLog('Error syncProductRatings: ' . $e->getMessage());
            return -1;
        }
    }

    public function parseProductsWithLanguages($idProductsArray, $languages = null)
    {
        try {
            $productsParsed = [];
            foreach ($idProductsArray as $idProduct) {

                $newProduct = $this->parseProduct($idProduct);

                if (!empty($newProduct)) {
                    $productsParsed[] = $newProduct;
                }
            }

            return $productsParsed;
        } catch (Exception $e) {
            $this->reviGeneralModel->errorLog('Error parseProductsWithLanguages: ' . $e->getMessage());
            return -1;
        }
    }

    public function sendProducts($products)
    {
        try {
            if (empty($products)) {
                return 0;
            }

            $result = $this->reviCurlModel->reviCURL('products', 'POST', [
                'products' => json_encode($products)
            ]);

            if (empty($result) || !$result->success) {
                $this->reviGeneralModel->errorLog('Error sendProducts reviCURL:result: ' . json_encode($result));
                return -1;
            }

            $numProductSent = 0;
            foreach ($products as $product) {
                $this->addReviProduct([
                    'id_product' => $product['id_external_product']
                ], date('Y-m-d H:i:s'));

                // En los multidioma de WPML, como recogemos el ID main siempre, guardamos el ID de WP del producto como sincronizado también
                if ($product['id_wp_product'] !== $product['id_external_product']) {
                    $this->addReviProduct([
                        'id_product' => $product['id_wp_product']
                    ], date('Y-m-d H:i:s'));
                }

                $numProductSent++;
            }

            return $numProductSent;
        } catch (Exception $e) {
            $this->reviGeneralModel->errorLog('Error sendProducts: ' . $e->getMessage());
            return -1;
        }
    }

    public function sendBatchPendingProducts($limit)
    {
        try {
            $numProductsLeft = (int) $this->getNumProductsLeft();

            if (!$numProductsLeft) {
                return 0;
            }

            $productIdsArray = $this->getProductsIdsToSend($limit);

            if (empty($productIdsArray)) {
                return 0;
            }

            $products = $this->parseProductsWithLanguages($productIdsArray);

            $numProductSent = $this->sendProducts($products);

            return $numProductSent;
        } catch (Exception $e) {
            $this->reviGeneralModel->errorLog('Error sendBatchPendingProducts: ' . $e->getMessage());
            return -1;
        }
    }
}
