<?php
if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

class revi_sync
{
    public $prefix;
    public $wpdb;

    public $reviDatabaseModel;
    public $reviGeneralModel;
    public $reviOrdersModel;
    public $reviProductsModel;
    public $subscription;

    public function __construct()
    {
        global $wpdb;
        $this->wpdb = $wpdb;
        $this->prefix = $wpdb->prefix;

        $this->reviDatabaseModel = new reviDatabaseModel();
        $this->reviGeneralModel = new reviGeneralModel();
        $this->reviOrdersModel = new reviOrdersModel();
        $this->reviProductsModel = new reviProductsModel();

        $this->reviGeneralModel->updateConfiguration();

        $this->subscription = get_option('REVI_SUBSCRIPTION');

        if (isset($_REQUEST['reset_data']) && $_REQUEST['reset_data'] == 1) {
            $this->reviDatabaseModel->truncateReviDatabase();
        }

        $limitOrders = 10;
        $limitProducts = 20;
        if (isset($_REQUEST['limit']) && !empty($_REQUEST['limit'])) {
            $limitOrders = $_REQUEST['limit'];
            $limitProducts = $_REQUEST['limit'];
        }

        $cycles = 10;
        if (isset($_REQUEST['cycles']) && !empty($_REQUEST['cycles'])) {
            $cycles = $_REQUEST['cycles'];
        }

        // UPDATE CONFIGURATION & DATABASE
        $this->reviDatabaseModel->checkAndCreateReviTables();
        $this->reviGeneralModel->updateConfiguration();

        $syncMessage = $this->sendBatchData($limitOrders, $limitProducts, $cycles);

        echo esc_html($syncMessage); // Escapado de la salida
    }

    public function sendBatchData($limitOrders, $limitProducts, $cycles)
    {
        try {
            $this->reviGeneralModel->sendModuleVersion();
            $this->reviProductsModel->sendNumProductsLeft();
            $this->reviProductsModel->syncProductRatings();

            $message = '';
            $countCycles = 0;

            do {
                // Sync Orders
                $numOrders = $this->reviOrdersModel->sendPendingOrders($limitOrders);
                $message .= "\n<br> $numOrders Orders Synced";

                // Sync Products
                $numProducts = $this->reviProductsModel->sendBatchPendingProducts($limitProducts);
                $message .= "\n<br> $numProducts Products Synced";

                if (!$numProducts && !$numOrders) {
                    break;
                }

                sleep(rand(0, 1));

                $countCycles++;
            } while ($countCycles < $cycles);

            $message .= "\n<br> $countCycles Sync cycles processed";

            return $message;
        } catch (Exception $e) {
            echo 'Error: ' . esc_html($e->getMessage());
            $this->reviGeneralModel->errorLog('Error sendBatchData: ' . $e->getMessage());
            return "\n Error sendBatchData: " . $e->getMessage();
        }
    }
}
