<?php

// WooCommerce actions and filters

if (!defined('ABSPATH')) {
    exit;
}

// If WooCommerce is not active, return
if (!defined('WOOCOMMERCE_ACTIVE') || !WOOCOMMERCE_ACTIVE) {
    return;
}

// Remove WooCommerce Reviews
add_action('init', 'revi_woocommerce_remove_reviews');
add_filter('woocommerce_structured_data_product', 'structured_data_product_nulled_wiped', 10, 2);

// Order status change
add_action('woocommerce_order_status_changed', 'revi_order_status_change', 10, 4);

// Order confirmation popup
add_action('woocommerce_thankyou', 'revi_popup_order_confirmation', 111, 1);


// Product
if (get_option('REVI_TAB_REVIEWS')) {

    add_filter('woocommerce_product_tabs', 'revi_product_tabs', 121);
} else {

    add_filter('woocommerce_after_single_product_summary', function () {

        revi_load_widget_html("product");

        if (get_option('REVI_SUBSCRIPTION') >= 2) {
            revi_load_widget_html("questions");
        }
    }, 121);
}

// Product Stars
if (get_option('REVI_PRODUCT_STARS_POSITION') == 1) {

    add_filter('woocommerce_before_add_to_cart_form', function () {

        revi_load_widget_html("stars");

        if (get_option('REVI_SUBSCRIPTION') >= 2) {
            revi_load_widget_html("questions_small");
        }
    }, 9);
} else {
    add_filter('woocommerce_single_product_summary', function () {

        revi_load_widget_html("stars");

        if (get_option('REVI_SUBSCRIPTION') >= 2) {
            revi_load_widget_html("questions_small");
        }
    }, 9);
}

// Product list
add_action('woocommerce_after_shop_loop_item_title', 'revi_product_list', 5);

// Product Metadata
if (get_option('REVI_PRODUCT_METADATA') && get_option('REVI_SUBSCRIPTION') >= 2) {
    add_action('wp_head', 'revi_schema_product');
}


function revi_product_tabs($tabs)
{
    global $post;
    $idProduct = $post->ID;
    $reviProductsModel = new reviProductsModel();
    $idProduct = $reviProductsModel->get_id_main_product($idProduct);
    $productInfo = $reviProductsModel->getReviProduct($idProduct);

    if (!(get_option('REVI_DISPLAY_WIDGET_WITHOUT_REVIEWS') == 0 && $productInfo->num_reviews == 0)) {
        $tabs['reviews'] = [
            'title' => __('Reviews', 'revi-io-customer-and-product-reviews'),
            'callback' => function () {
                revi_load_widget_html("product");
            },
        ];
    }

    if (get_option('REVI_DISPLAY_WIDGET_QUESTIONS_BIG') && get_option('REVI_SUBSCRIPTION') >= 2) {
        $tabs['questions'] = [
            'title' => __('Questions', 'revi-io-customer-and-product-reviews'),
            'callback' => function () {
                revi_load_widget_html("questions");
            },
        ];
    }

    return $tabs;
}


// PRODUCT WIDGETS
function revi_product_list()
{
    global $post;
    $idProduct = $post->ID;

    $reviProductsModel = new reviProductsModel();
    $productInfo = $reviProductsModel->getReviProduct($idProduct);

    $REVI_DISPLAY_PRODUCT_LIST_ALIGN = get_option('REVI_DISPLAY_PRODUCT_LIST_ALIGN');
    $REVI_DISPLAY_PRODUCT_LIST_EMPTY = get_option('REVI_DISPLAY_PRODUCT_LIST_EMPTY');
    $REVI_DISPLAY_PRODUCT_LIST_BLANK_SPACE = get_option('REVI_DISPLAY_PRODUCT_LIST_BLANK_SPACE') ?? 1;
    $REVI_DISPLAY_PRODUCT_LIST_TEXT = get_option('REVI_DISPLAY_PRODUCT_LIST_TEXT');

    include REVI_DIR . 'templates/hook/product_list.php';
}

// Remove WooCommerce Default Structured Data
function structured_data_product_nulled_wiped($markup, $product)
{
    return [];
}

// Remove WooCommerce Reviews
function revi_woocommerce_remove_reviews()
{
    $REVI_WOOCOMMERCE_REVIEWS = get_option('REVI_WOOCOMMERCE_REVIEWS');
    if (!isset($REVI_WOOCOMMERCE_REVIEWS) || !$REVI_WOOCOMMERCE_REVIEWS) {
        remove_post_type_support('product', 'comments');
        remove_action('woocommerce_after_shop_loop_item_title', 'woocommerce_template_loop_rating', 5);
    }
}

function revi_order_status_change($idOrder)
{
    try {
        if (empty($idOrder)) {
            return;
        }

        $reviOrdersModel = new reviOrdersModel();
        $reviOrdersModel->sendOrder($idOrder);
        $reviOrdersModel->sendOrderStatus($idOrder);

        $reviProductsModel = new reviProductsModel();
        $reviProductsModel->syncProductRatings();
    } catch (Exception $e) {
        $reviGeneralModel = new reviGeneralModel();
        $reviGeneralModel->errorLog('Error on revi_order_status_change: ' . $e->getMessage());
    }
}
