<?php

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

/*
* Plugin Name: Revi.io - Customer and Product Reviews
* Plugin URI: https://revi.io
* Description: Product ratings and Customer Reviews for WooCommerce ecommerce
* Version: 6.2.3
* Author: <a href="https://revi.io">Revi</a>
* Text Domain: revi-io-customer-and-product-reviews
* License: GPLv2 or later
*/

// Include required files
define('REVI_DIR', WP_PLUGIN_DIR . '/' . plugin_basename(dirname(__FILE__)) . '/');
define('REVI_PLUGIN_URL', plugin_dir_url(__FILE__));

require_once(plugin_dir_path(__FILE__) . 'includes/constants.php');
require_once(plugin_dir_path(__FILE__) . 'includes/functions.php');
require_once(plugin_dir_path(__FILE__) . 'includes/languages.php');
require_once(plugin_dir_path(__FILE__) . 'classes/reviDatabaseModel.php');
require_once(plugin_dir_path(__FILE__) . 'classes/reviGeneralModel.php');
require_once(plugin_dir_path(__FILE__) . 'classes/reviCurlModel.php');
require_once(plugin_dir_path(__FILE__) . 'classes/reviOrdersModel.php');
require_once(plugin_dir_path(__FILE__) . 'classes/reviProductsModel.php');
require_once(plugin_dir_path(__FILE__) . 'classes/reviWidgetsClass.php');
require_once(plugin_dir_path(__FILE__) . 'classes/reviProductsModel.php');

require_once(plugin_dir_path(__FILE__) . 'includes/admin.php');
require_once(plugin_dir_path(__FILE__) . 'includes/frontend.php');
require_once(plugin_dir_path(__FILE__) . 'includes/widgets.php');
require_once(plugin_dir_path(__FILE__) . 'includes/shortcodes.php');
require_once(plugin_dir_path(__FILE__) . 'includes/schema.php');
require_once(plugin_dir_path(__FILE__) . 'includes/woocommerce.php');

require_once(plugin_dir_path(__FILE__) . 'blocks/revi-block.php');


// Hooks
register_activation_hook(__FILE__, 'revi_install');
register_deactivation_hook(__FILE__, 'revi_uninstall');
add_action('plugins_loaded', 'revi_load_plugin_textdomain');

// Enqueue Frontend Styles and Scripts
add_action('wp_print_styles', 'revi_styles');
add_action('wp_enqueue_scripts', 'revi_scripts'); // Frontend

// Enqueue Admin Styles and Scripts
add_action('admin_enqueue_scripts', 'revi_admin_styles');

// Admin notice
add_action('admin_notices', 'my_theme_notice');

// Admin menu
add_action('admin_menu', 'revi_plugin_admin_add_page');

// Query Vars
add_filter('query_vars', 'revi_register_query_var');

// Load Controllers
add_filter('template_include', 'revi_template_include');



function revi_install()
{
    $reviDatabaseModel = new reviDatabaseModel();
    $reviDatabaseModel->checkAndCreateReviTables();
    revi_createContent();
}

function revi_uninstall()
{
    $reviDatabaseModel = new reviDatabaseModel();
    $reviDatabaseModel->deleteReviDatabase();
    delete_option('revi_options');
}

function revi_createContent()
{
    $plugin_data = get_plugin_data(WP_PLUGIN_DIR . '/revi-io-customer-and-product-reviews/revi.php');
    $plugin_version = $plugin_data['Version'];

    if (
        !update_option('REVI_MODULE_VERSION', $plugin_version)
        || !update_option('REVI_API_KEY', '')
        || !update_option('REVI_SUBSCRIPTION', '0')
        || !update_option('REVI_NOTIFICATIONS', array())
        || !update_option('REVI_SELECTED_STORE', 0)
        || !update_option('REVI_SELECTED_SHOPS', array(1))
        || !update_option('REVI_ORDER_STATUSES', array('wc-completed'))
        || !update_option('REVI_SELECTED_LANGUAGE', 'en')
        || !update_option('REVI_LANG', 'en')
        || !update_option('REVI_PRODUCT_METADATA', 0)
        || !update_option('REVI_TAB_REVIEWS', 0)
        || !update_option('REVI_PRODUCT_STARS_POSITION', 0)
        || !update_option('REVI_DISPLAY_WIDGET_FLOATING', 1)
        || !update_option('REVI_WOOCOMMERCE_REVIEWS', 0)
        || !update_option('REVI_SYNC_PRODUCT_DESCRIPTION', 0)
        || !update_option('REVI_DISPLAY_PRODUCT_LIST_ALIGN', 0)
        || !update_option('REVI_DISPLAY_PRODUCT_LIST_EMPTY', 0)
        || !update_option('REVI_DISPLAY_PRODUCT_LIST_BLANK_SPACE', 0)
        || !update_option('REVI_DISPLAY_PRODUCT_LIST_TEXT', 0)
    ) {
        return false;
    }
    return true;
}
